#!/usr/bin/env python
################################################################################
#   Program:    bbcode2html.py
#   Version:    0.9
#   Author:     x.seeks <x otday seeks atlay gmail otday com>
#   License:    GPL3, see LICENSE.txt
#
#   This scipt takes BBCode (used in forums, specifically, the PA forum) and
#   converts it, more or less, to HTML.  It's not pretty and quite inelegant,
#   but should serve as a good jumping-off point to mirroring an HTML copy of
#   any post you would like.
################################################################################
import os, sys, re

hex_colors = {'white':'#FFFFFF','silver':'#C0C0C0','gray':'#808080',
        'black':'#000000', 'red':'#FF0000', 'maroon':'#800000',
        'yellow':'#FFFF00','olive':'#808000','lime':'#00FF00',
        'green':'#008000','aqua':'#00FFFF','teal':'#008080','blue':'#0000FF',
        'navy':'#000080','fuchsia':'#FF00FF','purple':'#800080'}

def start_html(options):
    comment1_start = "<!--\n"
    comment1_end = "\n-->"
    css = 'style.css'

    if options['title'] != '':
        title = options['title']

    if options['css'] != '':
        comment1_start = ''
        css = options['css']
        comment1_end = ''

    html_start = """
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
<html xmlns="http://www.w3.org/1999/xhtml">


<head>
<meta http-equiv="Content-Type" content="text/html; charset=utf-8" />
<title>%s</title>

%s
<link rel="stylesheet" type="text/css" href="%s" />
<link rel="icon" href="favicon.ico" type="image/x-icon" />
   <link rel="shortcut icon" href="favicon.ico" type="image/x-icon" /> 
%s

</head>

<!-- After the body tag and until the end is generated by the script -->

<body>

    """ % (title, comment1_start, css, comment1_end)

    return html_start

def end_html():
    html_end = """

</body>

</html>
    """

    return html_end

def print_help():
    help_text = """

This script will read basic BBCode (used in forums) and spit out an HTML
document.  With no arguments (other than the file you're working on),
the script will print to stdout - you can use piping and redirects as normal
if you wish.

Options:

    -i [file] or --input=[file]
    Use this if you want to specify an input file.  Without this argument, the
    input defaults to the last argument given to the script, whatever it may
    be.

    -o [file] or --output=[file]
    This argument specifies an output file to write to.  With no such argument
    given, the program will print to standard output by default.

    -c [file] or --css=[file]
    This argument specifies a CSS file for use with the output.  By default,
    no CSS file is specified.

    -t "string" or --title="string"
    Specify a title for your page.

    """
    print (help_text)

def print_version():
    version = """

    bbcode2html.py, version 0.9

    """
    print (version)


def parse_file_name(post_arg):
    file_name = str(post_arg)
    extensions = ('.txt','.forum','.bbcode')
    for i in extensions:
        file_name = file_name.replace(i, '', -1)
    return file_name

def read_file(post_arg):
    post_file = open(post_arg, 'r')
    post = post_file.readlines()
    post_file.close()
    return post

def create_uppers(list_of_lowers):
    """
    Creates a list of uppercase instances from the key names in the replacement dictionary
    """
    ##  Create empty list to be populated later
    uppers = []

    ## Populate list
    for lower in list_of_lowers:
        if lower == '"]' or lower == "']":
            pass
        else:
            upper = lower.upper()
            uppers.append(upper)

    return uppers

def upper_to_lower(post, uppers):
    for line in post:
        line_index = post.index(line)
        for upper in uppers:
            if upper in line:
                line = line.replace(upper, upper.lower(), -1)
                post[line_index] = line

    return post

def replacement(post, replacement_dict):
    for line in post:
        line_index = post.index(line)
        edited = 0
        for key in replacement_dict:
            while key in line:
                line = line.replace(key, replacement_dict[key], -1)
                edited += 1
        if edited > 0:
            post[line_index] = line
    return post

def fix_color(line):
    part1 = line.partition('=')
    part2 = part1[-1].partition(']')
    color = part2[0]
    color2 = color.strip("'")
    color2 = color.strip('"')
    if color2 in hex_colors:
        new_color = hex_colors[color2]
        new_color += ';">'
    elif color2.lower() in hex_colors:
        new_color = hex_colors[color2.lower()]
        new_color += ';">'
    else:
        new_color = color2 +';">'

    #print ("YO!", color, color2, new_color)
    #sys.exit()
    line = (line.replace('[color=%s' % color,
        '<span style="color: %s' % new_color, -1))
    line = (line.replace('[COLOR=%s' % color,
        '<span style="COLOR: %s' % new_color, -1))
    line = (line.replace('[colour=%s' % color,
        '<span style="colour: %s' % new_color, -1))
    line = (line.replace('[colour=%s' % color,
        '<span style="colour: %s' % new_color, -1))

    line = line.replace('[/color]', '</span>', -1)
    line = line.replace('[/COLOR]', '</span>', -1)
    line = line.replace('[/colour]', '</span>', -1)
    line = line.replace('[/colour]', '</span>', -1)

    return line


def fix_quote(line):
    index1 = line.find('=')
    part1 = line.partition(line[index1+1])
    part2 = line.partition(part1[-1].partition(']'))
    replace_me = part2[0]

    line = line.replace(part1[0], '<blockquote>\n<p>\n', -1)
    line = line.replace(replace_me, '', -1)
    line = line.replace(']', '', -1)
    line = line.replace('[/quote]','\n</p>\n</blockquote>', -1)
    line = line.replace('[/QUOTE]','\n</p>\n</blockquote>', -1)

    return line

def fix_font(line):
    part1 = line.partition('FONT=')
    part1_l = line.partition('font=')
    part2 = part1[-1].partition(']')
    part2_l = part1_l[-1].partition(']')


    line = line.replace('[font=', "", -1)
    line = line.replace('[FONT=', "", -1)
    line = line.replace(part2[0], '', -1)
    line = line.replace(part2_l[0], '', -1)
    line = line.replace('[/font]','', -1)
    line = line.replace('[/FONT]','', -1)

    return line

def fix_size(line):
    part1 = line.partition('=')
    part2 = part1[-1].partition(']')
    size = part2[0]
    size = size.replace('"','',-1)
    size = size.replace("'",'', -1)
    try:
        size_number = int(size)
        size_number *= 4
        size = str(size_number)
    except ValueError:
        pass
    size += 'pt'

    line = (line.replace('[size=%s' % part2[0],
        '<span style="font-size: %s">' % size, -1))
    line = (line.replace('[SIZE=%s' % part2[0],
        '<span style="font-size: %s">' % size, -1))
    line = line.replace('[/size]','</span>', -1)
    line = line.replace('[/SIZE]','</span>', -1)
    #line = line.replace(part2[1],'',1)

    return line

def bbcode_to_html(post):
    """
    This is the long and tedious section of the script.  It tries to find a number of BBCode tags in the post and replaces them with their HTML counterparts.
    """
    ## Create a working copy of the post, just for fun
    newpost = post
    replacement_dict = {"[b]":"<strong>", "[/b]":"</strong>", "[i]":"<em>", 
            "[/i]":"</em>", "[u]":"<ins>", "[/u]":"</ins>", "[s]":"<del>",
            "[/s]":"</del>", "[url]":'<a href=', "[url=":'<a href=',
            '"]':'">', "']":"'>", "[/url]":"</a>", "[img]":'<img src="',
            "[/img]":'"/>', "[table]":"<table>","[/table]":"</table>",
            "[tr]":"<tr>", "[/tr]":"</tr>", "[td]":"<td>", "[/td]":"</td>",
            "[quote]":"<blockquote>\n<p>\n",
            '[/quote]':'\n</p>\n</blockquote>', '[spoiler]':'\n',
            '[size=':"<span style=font-size:",'[/size]':'</span>',
            '[/spoiler]':'\n'}

    ## Fix 'color' attributes in a line
    c1 = '[color='
    c2 = '[colour='
    for line in newpost:
        line_index = newpost.index(line)
        if c1 in line or c1.upper() in line or c2 in line or c2.upper() in line:
            line = fix_color(line)
            newpost[line_index] = line

    ##  Fix 'quote' attributes in a line
    q = '[quote='
    for line in newpost:
        line_index = newpost.index(line)
        if q in line or q.upper() in line:
            line = fix_quote(line)
            newpost[line_index] = line

    ##  Fix 'font' attributes in a line
    f = '[font='
    for line in newpost:
        line_index = newpost.index(line)
        if f in line or f.upper() in line:
            line = fix_font(line)
            newpost[line_index] = line

    ##  Fix 'size' attributes
    s = '[size='
    for line in newpost:
        line_index = newpost.index(line)
        if s in line or s.upper() in line:
            line = fix_size(line)
            newpost[line_index] = line

    ##  Create a list of uppercase instances corresponding to the key names
    ##  in the replacement dictionary
    uppers = create_uppers(replacement_dict)

    ### Deal with uppercase letters
    post = upper_to_lower(newpost, uppers)

    ### Deal with the rest
    post = replacement(post, replacement_dict)

    ### Clean up
    last_dict = {']':'', '\n':'<NEWLINE>', '<NEWLINE>':'\n<br/>\n'}
    post = replacement(post, last_dict)

    return post


def write_file(post, options, html_start, html_end):
    if options['output'] == '':
        document = []
        document.append(html_start)
        for line in post:
            document.append(line)
        document.append(html_end)
        for line in document:
            print (line)
    elif options['output'] != '':
        html_file = open(options['output'], 'w')
        html_file.write(html_start)
        for line in post:
            html_file.write(line)
        html_file.write(html_end)
        html_file.close()

def parse_args(file_name):
    args = sys.argv
    options = {'input':args[1],'output':'','css':'','title':file_name}
    arg_tup = ('-i','--input=','-o','--output=','-h','--help', '-c',
            '--css=','-t','--title=', '-v','--version')
    for arg in arg_tup:
        if arg in args:
            if arg == '-h' or arg == '--help':
                print_help()
                sys.exit()
            elif arg == '-v' or arg == '--version':
                print_version()
                sys.exit()
            elif arg == '-i':
                arg_index = args.index(arg)
                options['input'] = args[arg_index + 1]
            elif arg == '-o':
                arg_index = args.index(arg)
                options['output'] = args[arg_index + 1]
            elif arg == '-c':
                arg_index = args.index(arg)
                options['css'] = args[arg_index + 1]
            elif arg == '-t':
                arg_index = args.index(arg)
                options['title'] = args[arg_index + 1]
    for arg in args:
        if '--input=' in arg:
            new_arg = arg.replace('--input=', '', 1)
            options['input'] = new_arg
        if '--output=' in arg:
            new_arg = arg.replace('--output=', '', 1)
            options['output'] = new_arg
        if '--css=' in arg:
            new_arg = arg.replace('--css=', '', 1)
            options['css'] = new_arg
        if '--title=' in arg:
            new_arg = arg.replace('--title=', '', 1)
            options['title'] = new_arg

    return options


def main():
    first_arg = sys.argv[-1]
    file_name = parse_file_name(first_arg)
    options = parse_args(file_name)
    html_start = start_html(options)
    html_end = end_html()
    post = read_file(options['input'])
    post = bbcode_to_html(post)
    write_file(post, options, html_start, html_end)

if __name__ == '__main__':
    main()
